#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define input(directions,pin) (directions &= (~pin)) // set port direction for input 
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set

#define bit_delay_time 100 // bit delay for 9600 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pin_in (1 << PA0)
#define serial_pin_out (1 << PA1)
#define serial_pins PINA

#define nodeA '0'
#define nodeB '1'
#define nodeAon '2'
#define nodeAoff '3'
#define nodeBon '4'
#define nodeBoff '5'

void get_char(char *rxbyte) {
	//
	// read character into rxbyte on pins pin
	//	 assumes line driver (inverts bits)
	//
	*rxbyte = 0;
	while (pin_test(serial_pins,serial_pin_in))
		//
		// wait for start bit
		//
		;
	//
	// delay to middle of first data bit
	//
	half_bit_delay();
	bit_delay();
	//
	// unrolled loop to read data bits
	//
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 0);
	else
		*rxbyte |= (0 << 0);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 1);
	else
		*rxbyte |= (0 << 1);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 2);
	else
		*rxbyte |= (0 << 2);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 3);
	else
		*rxbyte |= (0 << 3);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 4);
	else
		*rxbyte |= (0 << 4);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 5);
	else
		*rxbyte |= (0 << 5);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 6);
	else
		*rxbyte |= (0 << 6);
	bit_delay();
	if pin_test(serial_pins,serial_pin_in)
		*rxbyte |= (1 << 7);
	else
		*rxbyte |= (0 << 7);
	//
	// wait for stop bit
	//
	bit_delay();
	half_bit_delay();
	//
	// char delay
	//
	bit_delay();
}

void put_char(char txchar) {
	//
	// send character in txchar on port pin
	//	 assumes line driver (inverts bits)
	//
	// start bit
	//
	clear(serial_port,serial_pin_out);
	bit_delay();
	//
	// unrolled loop to write data bits
	//
	if bit_test(txchar,0)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,1)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,2)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,3)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,4)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,5)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,6)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	if bit_test(txchar,7)
		set(serial_port,serial_pin_out);
	else
		clear(serial_port,serial_pin_out);
	bit_delay();
	//
	// stop bit
	//
	set(serial_port,serial_pin_out);
	bit_delay();
	//
	// char delay
	//
	bit_delay();
}

int main(void) {
	// set clock divider to /1
	CLKPR = (1 << CLKPCE);
	CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
	
	// inititalize pins
	set(serial_port, serial_pin_out);
	output(serial_direction, serial_pin_out);
	input(serial_direction, serial_pin_in);
	
	_delay_ms(1000);
	
	static char chr;
	
	// wait for interrupt
	while (1) {
		put_char(nodeA);
		get_char(&chr);
		if (chr == nodeAon)
			put_char(nodeBon);
		else if (chr == nodeAoff)
			put_char(nodeBoff);
		put_char(nodeB);
		get_char(&chr);
		if (chr == nodeBon)
			put_char(nodeAon);
		else if (chr == nodeBoff)
			put_char(nodeAoff);
	}
}
